//
// Computergraphik II
// Prof. Dr. Juergen Doellner
// Wintersemester 2001/2002
// 
// Rahmenprogramm fuer Aufgabenzettel 3
//
// Autoren: Florian Kirsch (kirsch@hpi.uni-potsdam.de)
//          Marc Nienhaus (nienhaus@hpi.uni-potsdam.de)
//          Juergen Doellner (doellner@hpi.uni-potsdam.de)
//

#include "cghsvcolors.h"


CGHSVColors::CGHSVColors(int width, int height) : raster_(width,height) {
    mode_ = DrawGrid;
    run_ = false;
    angle_ = 30;

	clrFrom = Color(1.0, 0.0, 0.5);
	clrTo   = Color(0.45, 0.55, 0.4);
}

void CGHSVColors::onInit() {
    glClearColor(1, 1, 1, 1);
	// anti-aliased points
//	glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
//	glEnable(GL_POINT_SMOOTH);
//	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void CGHSVColors::onDraw() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

//    if (mode_ == DrawGrid) {

	// color banner
	glViewport(0, 0, winWidth_-1, winHeight_/10);

        glDisable(GL_DEPTH_TEST);

        glMatrixMode(GL_PROJECTION);
        gluOrtho2D(0, raster_.width(), 0, raster_.height());
        glMatrixMode(GL_MODELVIEW);

        // Hier 2D-Raster zeichnen
		raster_.clear();
		const int colors = raster_.width();
		for (int x=0; x<colors; x++)
		{
			// RGB
			raster_.setPixel(x,0,Color::InterpolateRGB(clrFrom, clrTo, x/(double)colors));
			// HSV +
			raster_.setPixel(x,1,Color::InterpolateHSV(clrFrom, clrTo, x/(double)colors, true));
			// HSV -
			raster_.setPixel(x,2,Color::InterpolateHSV(clrFrom, clrTo, x/(double)colors, false));
		}
		raster_.draw();

//    } else if (mode_ == DrawCube) {

	// cube
	glViewport(0, winHeight_/10, winWidth_-1, winHeight_*0.9);

        glEnable(GL_DEPTH_TEST);

        glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
        gluPerspective(30.0,1.0,2,2000.0);   
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        gluLookAt(0.0, 0.0, 4.0,  // from (0,0,4) 
                  0.0, 0.0, 0.0,  // to (0,0,0) 
                  0.0, 1.0, 0.);  // up

        glRotatef(angle_, 0.0, 1.0, 0.0);
        glColor4d(0.0, 0.0, 0.0, 1.0);
        glutWireCube(1.0);
        glTranslatef(-0.5, -0.5, -0.5);

        // Hier 3D-Punkte zeichnen

		glPointSize(3);
		glBegin(GL_POINTS);
		for (x=0; x<SHOWNCOLORS; x++)
		{
			Color clrInterpolate;

			// RGB
			clrInterpolate = Color::InterpolateRGB(clrFrom, clrTo, x/(double)SHOWNCOLORS);
			glColor3d(clrInterpolate[0], clrInterpolate[1], clrInterpolate[2]);
			glVertex3d(clrInterpolate[0], clrInterpolate[1], clrInterpolate[2]);

			// HSV +
			clrInterpolate = Color::InterpolateHSV(clrFrom, clrTo, x/(double)SHOWNCOLORS, true);
			glColor3d(clrInterpolate[0], clrInterpolate[1], clrInterpolate[2]);
			glVertex3d(clrInterpolate[0], clrInterpolate[1], clrInterpolate[2]);
		
			// HSV -
			clrInterpolate = Color::InterpolateHSV(clrFrom, clrTo, x/(double)SHOWNCOLORS, false);
			glColor3d(clrInterpolate[0], clrInterpolate[1], clrInterpolate[2]);
			glVertex3d(clrInterpolate[0], clrInterpolate[1], clrInterpolate[2]);
		}
		glEnd();

//    }

    swapBuffers();
}

void CGHSVColors::onSize(unsigned int newWidth,unsigned int newHeight) {
    winWidth_ = newWidth;
    winHeight_ = newHeight;
    glViewport(0, 0, newWidth - 1, newHeight - 1);
}

void CGHSVColors::onButton(MouseButton button, int x, int y) {
    onDraw();
}

void CGHSVColors::onIdle() {  
    if (run_) {
        angle_ += 1.0;
        onDraw();
    }
}

void CGHSVColors::onKey(unsigned char key) {
    switch (key) {
    case 27: { exit(0); break; }                        
//    case 'g': mode_ = DrawGrid; break;
//    case 'c': mode_ = DrawCube; break;

		// change "from" color
	case 'a': if (clrFrom[0] <= 0.95) clrFrom[0] += 0.05; break;
	case 'y': if (clrFrom[0] >= 0.05) clrFrom[0] -= 0.05; break;
	case 's': if (clrFrom[1] <= 0.95) clrFrom[1] += 0.05; break;
	case 'x': if (clrFrom[1] >= 0.05) clrFrom[1] -= 0.05; break;
	case 'd': if (clrFrom[2] <= 0.95) clrFrom[2] += 0.05; break;
	case 'c': if (clrFrom[2] >= 0.05) clrFrom[2] -= 0.05; break;
		// change "to" color
	case 'f': if (clrTo  [0] <= 0.95) clrTo  [0] += 0.05; break;
	case 'v': if (clrTo  [0] >= 0.05) clrTo  [0] -= 0.05; break;
	case 'g': if (clrTo  [1] <= 0.95) clrTo  [1] += 0.05; break;
	case 'b': if (clrTo  [1] >= 0.05) clrTo  [1] -= 0.05; break;
	case 'h': if (clrTo  [2] <= 0.95) clrTo  [2] += 0.05; break;
	case 'n': if (clrTo  [2] >= 0.05) clrTo  [2] -= 0.05; break;

    case ' ': run_ = !run_; break;
    }
    onDraw();
}

int main(int argc, char* argv[]) {
    CGHSVColors hsvcolors(32, 3);

    cout << "Tastenbelegung:" << endl
		 << "ESC        Programm beenden" << endl
		 << "Leertaste  Objekt drehen" << endl
		 << "a          Rotanteil  der Startfarbe erhhen" << endl
		 << "y          Rotanteil  der Startfarbe verringern" << endl
		 << "s          Grnanteil der Startfarbe erhhen" << endl
		 << "x          Grnanteil der Startfarbe verringern" << endl
		 << "d          Blauanteil der Startfarbe erhhen" << endl
		 << "c          Blauanteil der Startfarbe verringern" << endl
		 << "f          Rotanteil  der Zielfarbe  erhhen" << endl
		 << "v          Rotanteil  der Zielfarbe  verringern" << endl
		 << "g          Grnanteil der Zielfarbe  erhhen" << endl
		 << "b          Grnanteil der Zielfarbe  verringern" << endl
		 << "h          Blauanteil der Zielfarbe  erhhen" << endl
		 << "n          Blauanteil der Zielfarbe  verringern" << endl;

    hsvcolors.start("Stephan Brumme, 702544", true, 400, 400);

    return(0);
}
